"""
*************************************************************************
*************************************************************************
                    RASPBERRY PI COMPUTER BOARD
                     WINDSPEED TRACKER PROJECT
*************************************************************************
*************************************************************************

Date: 9/30/15   
File: RaspberryWindspeed_py   

This Python3 program plots the maximum windspeed values from an anemometer
on an LCD screen, at the rate of 1 new plot/minute, 1440 plots/day.

Notes:
1. During a full system bootup, the plotting does not start for several
minutes because the GPS receiver must first acquire the satellites. Plus,
the Arduino does not send data for the first 1-2 minutes and the Raspberry
Pi requires 1-2 minutes to initially fill the screen.

2. The time delay prior to expanding the screen(ALT-F11) may require
an increase from 0.5 sec to a longer time, up to 0.9 sec if the screen
does not expand correctly.

3. This program was written in 2015 by David Goodsell.

Please see gnuplot and xdotool Copyright notices below.

"""

#!/usr/bin/python3

import os            #Python module
import math          #Python module
import time          #Python module
import subprocess    #Python module
import serial        #Requires installation 
import pygame        #Requires installation

def playSound(filename):
    pygame.mixer.music.load(filename)
    pygame.mixer.music.play()
    
pygame.init()
port = '/dev/ttyAMA0'  #GPIO Pin 15(UART_RXD)
s = serial.Serial(port,115200,timeout=1)          
s.flushInput()

while True:
    s.flushInput()
    data = s.readline()  
    if len(data) >= 100:
     
       #GENERATES DATA FILE FOR WINDSPEED PLOT
       #*************************************** 
       thou = ((data[24]-48)*1000)
       hund = ((data[25]-48)*100)
       tens = ((data[27]-48)*10)
       units = (data[28]-48)
       LatestPlotPoint = ((30 + ((thou + hund + tens + units)*3))+6)
       if (LatestPlotPoint >= 4800):
          LatestPlotPoint = 1
       f = open('d.dat','w')
       for k in range (30,LatestPlotPoint):
          data1 = chr(data[k])
          if data1 == (";"):
             number = chr(data[k-2]) + chr(data[k-1])
             j = k/3
             j = int(j)-10
             data2 = "{}{}{}\n".format(j," ",number)
             f.write (data2)
             k=k+3
       f.close()

       #GENERATES CONFIG FILE FOR ALL LABELS ON PLOT
       #*********************************************
       string1 = ""
       string2 = ""
       f = open('plotconfig.txt','w')
       for m in range (4353,5800):    
          data1 = chr(data[m])
          if data1 != ";":
             string1 = string1 + chr((data[m]))
          else:
             if string1 != "00":
                string2 = "{}\n".format(string1)   
                f.write (string2)
             string1 = ""
             m = m + 1
       f.close()

       #GENERATES DATA FILE FOR maxToday DOTTED LINE
       #*********************************************
       #Note: This routine generates an extended dotted error bar,
       #as a substitute for a plotted dotted line
       f = open('dashline.dat','w')
       tens = ((data[(4353+25)]-48)*10)
       units = (data[(4353+26)]-48)
       MaxDottedLine = (tens + units)                        
       data2 = "{}{}{}\n".format(0," ",MaxDottedLine)
       f.write (data2)
       f.close()
                       
       #USES THE ABOVE DATA AND CONFIG FILES TO PLOT A GRAPH
       #*****************************************************
       proc = subprocess.Popen(['gnuplot plotconfig.txt','-p'],
          shell = True,
          stdin=subprocess.PIPE,
          )

       #EXPANDS THE SMALL PLOT WINDOW TO FULL SCREEN, USING ALT-F11
       #************************************************************
       time.sleep(.5)
       proc = subprocess.Popen(['xdotool key alt+F11'],
          shell = True,
          stdin=subprocess.PIPE,
          )

       #PLAYS TWO BEEPS WHENEVER MAX WINDSPEED GOES UP
       #***********************************************
       #if AudioAlertFlag from Arduino is true(data[22]=ASCII 8),
       #then play sound
       if data[22] == 56:
          playSound('Ping1a.wav')
          time.sleep(.9)
          playSound('Ping1a.wav')
                         
    s.flushInput()
    

"""
COPYRIGHT NOTICES:

The "gnuplot" software application is Copyright 1986-1993,2004 by
Thomas Williams and Colin Kelley. Permission to use, copy, and 
distribute this software and its documentation for any purpose with
or without fee is hereby granted.

The "xdtool" software application is Copyright(c)2007,2008,2009 by
Jordan Sissel. All rights reserved.
Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met: 
    * Redistributions of source code must retain the above copyright 
      notice, this list of conditions and the following disclaimer. 
    * Redistributions in binary form must reproduce the above copyright 
      notice, this list of conditions and the following disclaimer in the 
      documentation and/or other materials provided with the distribution. 
    * Neither the name of the Jordan Sissel nor the names of its contributors 
      may be used to endorse or promote products derived from this software 
      without specific prior written permission. 
 
THIS SOFTWARE IS PROVIDED BY JORDAN SISSEL ``AS IS'' AND ANY 
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL JORDAN SISSEL BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 


"""



                         
